<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <link rel="icon" href="favicon.ico" />
  <link rel="stylesheet" href="style.css">
  <title>{{ _app_title }}</title>
</head>

<body>
  <div id="app"></div>
</body>

<script type="module" src="app.js"></script>
</html>